exception Exn of string

type section = Code | Dlpt | Dlls | Prim | Data | Symb | Crcs | Dbug

type t = (section * int * int) list

let magic_str = "Caml1999X008"

let find_section index name =
  let (_, offset, length) = List.find (fun (n, _, _) -> name = n) index in
    (offset, length)
;;

let string_of_section section =
  match section with
    | Code -> "CODE" | Dlpt -> "DLPT" | Dlls -> "DLLS" | Prim -> "PRIM"
    | Data -> "DATA" | Symb -> "SYMB" | Crcs -> "CRCS" | Dbug -> "DBUG"
;;

let print oc index =
  let print_descr (section, offset, length) =
    Printf.fprintf oc "%s %8d %8d\n" (string_of_section section) offset length
  in
    List.iter print_descr index
;;

let export oc index =
  let rec f rest size =
    match rest with
      | (section, _, length) :: tl ->
	  output_string oc (string_of_section section);
	  output_binary_int oc length;
	  f tl (size + 1);
      | [] ->
	  output_binary_int oc size;
	  output_string oc magic_str;
  in
    f index 0
;;
